# -*- coding: utf-8 -*-

# -- Path setup --------------------------------------------------------------
import os
from urllib.parse import urljoin

from sphinx_revealjs.utils import get_revealjs_path

# -- Project information -----------------------------------------------------
project = "sphinx-revealjs"
copyright = "2018, Kazuya Takei"
author = "Kazuya Takei"
version = ""
release = "2018.10"

# -- General configuration ---------------------------------------------------
extensions = [
    "atsphinx.mini18n",
    "oembedpy.ext.sphinx",
    "rst_package_refs.sphinx",
    "sphinx.ext.mathjax",
    "sphinx.ext.todo",
    "sphinx_revealjs",
    "sphinx_revealjs.ext.footnotes",
    "sphinx_revealjs.ext.screenshot",
    "sphinxcontrib.budoux",
    "sphinxcontrib.gtagjs",
    "sphinxcontrib.sass",
    "sphinxext.opengraph",
]
templates_path = ["_templates"]
source_suffix = ".rst"
master_doc = "index"
language = "en"
locale_dirs = ["_locales"]
exclude_patterns = [".venv", "_build", "Thumbs.db", ".DS_Store", "_sections"]
pygments_style = None

# -- Options for HTML output -------------------------------------------------
html_theme = "alabaster"
html_static_path = ["_static"]

# -- Options for Reveal.js output ---------------------------------------------
revealjs_html_theme = "revealjs-simple"
revealjs_static_path = ["_static"]
revealjs_style_theme = "custom.css"
revealjs_script_conf = {
    "controls": True,
    "progress": True,
    "hash": True,
    "center": True,
    "transition": "slide",
    "customcontrols": {
        "controls": [
            {
                "icon": "EN",
                "action": "location.href = '/en/';",
            },
            {
                "icon": "JA",
                "action": "location.href = '/ja/';",
            },
        ]
    },
}
revealjs_script_plugins = [
    {
        "name": "RevealNotes",
        "src": "revealjs/plugin/notes/notes.js",
    },
    {
        "name": "RevealHighlight",
        "src": "revealjs/plugin/highlight/highlight.js",
    },
    {
        "name": "RevealMath",
        "src": "revealjs/plugin/math/math.js",
    },
    {
        "name": "RevealCustomControls",
        "src": "https://cdn.jsdelivr.net/npm/reveal.js-plugins@latest/customcontrols/plugin.js",
    },
]
revealjs_css_files = [
    "revealjs/plugin/highlight/zenburn.css",
    "https://cdn.jsdelivr.net/npm/reveal.js-plugins@latest/customcontrols/style.css",
]
revealjs_notes_from_comments = True

# -- Options for extensions --------------------------------------------------
# - sphinx.ext.todo
todo_include_todos = True
# - sphinxcontrib.gtagjs
if "GTAGJS_IDS" in os.environ:
    gtagjs_ids = os.environ["GTAGJS_IDS"].split(",")
# - sphinxcontrib.budoux
budoux_targets = ["h1", "h2", "h3"]
# - sphinxcontrib.sass
sass_src_dir = "_sass"
sass_out_dir = "_static"
sass_targets = {"custom.scss": "custom.css"}
sass_include_paths = [
    get_revealjs_path() / "css" / "theme",
]
# sphinxext.opengraph
ogp_site_url = os.environ.get("DEMO_URL_BASE", "http://localhost:8000/")
ogp_custom_meta_tags = [
    '<meta name="twitter:card" content="summary_large_image" />',
    '<meta name="twitter:site" content="@attakei" />',
]

# atsphinx-mini18n
mini18n_default_language = "en"
mini18n_support_languages = ["en", "ja"]
mini18n_basepath = "/sphinx-revealjs/"


def update_ogp(app, config):
    print(config.ogp_site_url, config.language)
    config.ogp_site_url = urljoin(config.ogp_site_url, f"{config.language}/")


def _add_navigation_for_mini18n(app, config):
    config.revealjs_script_conf["customcontrols"] = {
        "controls": [
            {
                "icon": lang.upper(),
                "action": f"location.href = '{config.mini18n_basepath}{lang}/';",
            }
            for lang in config.mini18n_support_languages
        ]
    }


def setup(app):
    app.connect("config-inited", update_ogp)
    app.connect("config-inited", _add_navigation_for_mini18n)



"""Test cases for ``revealjs-section`` directive."""

import io
from textwrap import dedent

import pytest
from sphinx.testing import restructuredtext
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="default")
def test_render_section1_bg(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-section.html")
    section_tag = soup.h1.parent
    assert "data-background-color" in section_tag.attrs
    assert section_tag["data-background-color"] == "#000001"


@pytest.mark.sphinx("revealjs", testroot="default")
def test_render_section2_bg(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-section.html")
    section_tag = soup.h2.parent
    assert "data-background-color" in section_tag.attrs
    assert section_tag["data-background-color"] == "#000101"


@pytest.mark.sphinx("revealjs", testroot="default")
def test_no_render_parent_bg(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-section.html")
    section_tag = soup.h2.parent.parent
    assert "data-background-color" not in section_tag.attrs


@pytest.mark.sphinx("revealjs", testroot="default")
def test_render_vertical_bg_vslide1(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-vertical-1.html")
    section_tag = soup.h1.parent.parent
    assert "data-background-color" in section_tag.attrs
    assert section_tag["data-background-color"] == "#000001"


@pytest.mark.sphinx("revealjs", testroot="default")
def test_render_vertical_bg_vslide2(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-vertical-2.html")
    section_tag = soup.h2.parent.parent
    assert "data-background-color" in section_tag.attrs
    assert section_tag["data-background-color"] == "#000001"


@pytest.mark.sphinx("revealjs", testroot="default")
def test_render_custom_attributes(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-section-having-custom-attributes.html")
    section_tag = soup.h2.parent
    assert "data-markdown" in section_tag.attrs
    assert section_tag["data-markdown"] == ""


class TestForRevealjsSection:
    @pytest.mark.sphinx("revealjs", testroot="default")
    def test_csutom_params(self, app: SphinxTestApp, warning: io.StringIO):
        text = dedent(
            """
        .. revealjs-section::
           :data-x:
        """
        )
        doctree = restructuredtext.parse(app, text)
        assert 'ERROR: Error in "revealjs-section" directive:' not in warning.getvalue()
        assert "data-x" in doctree.children[0].attributes

    @pytest.mark.sphinx("revealjs", testroot="default")
    def test_invalid_section_params(self, app: SphinxTestApp, warning: io.StringIO):
        text = dedent(
            """
        .. revealjs-section::
           :x-data:
        """
        )
        doctree = restructuredtext.parse(app, text)
        assert 'ERROR: Error in "revealjs-section" directive:' in warning.getvalue()
        assert 'unknown option: "x-data"' in warning.getvalue()
        assert "x-data" not in doctree.attributes


class TestForRevealjsBreak:
    @pytest.mark.sphinx("revealjs", testroot="default")
    def test_csutom_params(self, app: SphinxTestApp, warning: io.StringIO):
        text = dedent(
            """
        .. revealjs-break::
           :data-x:
        """
        )
        doctree = restructuredtext.parse(app, text)
        assert 'ERROR: Error in "revealjs-break" directive:' not in warning.getvalue()
        assert "data-x" in doctree.children[0].attributes

    @pytest.mark.sphinx("revealjs", testroot="default")
    def test_invalid_section_params(self, app: SphinxTestApp, warning: io.StringIO):
        text = dedent(
            """
        .. revealjs-break::
           :x-data:
        """
        )
        doctree = restructuredtext.parse(app, text)
        assert 'ERROR: Error in "revealjs-break" directive:' in warning.getvalue()
        assert 'unknown option: "x-data"' in warning.getvalue()
        assert "x-data" not in doctree.attributes


class TestForRevealjsVertical:
    @pytest.mark.sphinx("revealjs", testroot="default")
    def test_csutom_params(self, app: SphinxTestApp, warning: io.StringIO):
        text = dedent(
            """
        .. revealjs-vertical::
           :data-x:
        """
        )
        doctree = restructuredtext.parse(app, text)
        assert (
            'ERROR: Error in "revealjs-vertical" directive:' not in warning.getvalue()
        )
        assert "data-x" in doctree.children[0].attributes

    @pytest.mark.sphinx("revealjs", testroot="default")
    def test_invalid_section_params(self, app: SphinxTestApp, warning: io.StringIO):
        text = dedent(
            """
        .. revealjs-vertical::
           :x-data:
        """
        )
        doctree = restructuredtext.parse(app, text)
        assert 'ERROR: Error in "revealjs-vertical" directive:' in warning.getvalue()
        assert 'unknown option: "x-data"' in warning.getvalue()
        assert "x-data" not in doctree.attributes



"""Configuration for pytest."""

import inspect
from pathlib import Path

import pytest
from _pytest.config import Config
from packaging import version
from sphinx import deprecation

from sphinx_revealjs.deprecated import _get_sphinx_version

pytest_plugins = "sphinx.testing.fixtures"
collect_ignore = ["roots"]


def pytest_configure(config: Config):  # noqa
    for name, klass in inspect.getmembers(deprecation, inspect.isclass):
        if not issubclass(klass, DeprecationWarning):
            continue
        config.addinivalue_line(
            "filterwarnings", f"error::{klass.__module__}.{klass.__name__}"
        )


@pytest.fixture(scope="session")
def rootdir():
    """Set root directory to use testing sphinx project."""
    current_ver = _get_sphinx_version()
    delimter_ver = version.parse("7.2.0")
    if current_ver < delimter_ver:
        from sphinx.testing.path import path

        return path(__file__).parent.abspath() / "roots"
    return Path(__file__).parent.resolve() / "roots"



"""Custom write module."""

from docutils.nodes import (  # type: ignore
    Element,
    SkipNode,
    comment,
    literal_block,
    section,
    title,
)
from sphinx.writers.html5 import HTML5Translator

from .nodes import revealjs_break, revealjs_section, revealjs_slide, revealjs_vertical


class RevealjsSlideTranslator(HTML5Translator):
    """Translate Reveal.js HTML class."""

    permalink_text = False

    def __init__(self, builder, *args, **kwds):  # noqa: D107
        super().__init__(builder, *args, **kwds)
        self.builder.add_permalinks = False
        self._nest_step = 0

    def visit_section(self, node: section):
        """Begin ``section`` node.

        - Find first ``revealjs_section`` node and build attributes string.
        - When enter next section, nest level.
        """
        self.section_level += 1
        if self.section_level >= 4:
            return
        idx = node.first_child_matching_class(revealjs_section)
        attrs = "" if idx is None else node.children[idx].attributes_str()  # type: ignore[attr-defined]
        if node.attributes.get("ids") and self.config.revealjs_use_section_ids:
            attrs += ' id="{}"'.format(node.attributes["ids"][-1])

        if self._nest_step > 0:
            self.body.append("</section>\n" * self._nest_step)
            self._nest_step = 0
        if self.section_level == 1:
            self._nest_step = 2
            s_idx = node.first_child_matching_class(revealjs_slide)
            self.builder.revealjs_slide = (  # type: ignore[attr-defined]
                None if s_idx is None else node.children[s_idx]
            )
        elif self.section_level == 2:
            self._nest_step = 1

        if self._nest_step > 0:
            v_idx = node.first_child_matching_class(revealjs_vertical)
            v_attrs = "" if v_idx is None else node.children[v_idx].attributes_str()  # type: ignore[attr-defined]
            self.body.append(f"<section {v_attrs}>\n")
        self.body.append(f"<section {attrs}>\n")

    def depart_section(self, node: section):
        """End ``section``.

        Dedent section level
        """
        self.section_level -= 1
        # NOTE:: When level changes 1 to 0, tag is closed by proc of ``_nest_step``.
        if self.section_level == 0:
            return
        # NOTE:: Reveal.js is max section level is 2.
        if self.section_level > 2:
            return
        self.body.append("</section>\n")

    def visit_comment(self, node: Element):
        """Begin ``comment`` node.

        comment node render as speaker note.
        """
        if self.builder.app.config.revealjs_notes_from_comments:
            self.body.append('<aside class="notes">\n')
        else:
            raise SkipNode

    def depart_comment(self, node: comment):
        """End ``comment`` node.

        Close speaker note.
        """
        if self.builder.app.config.revealjs_notes_from_comments:
            self.body.append("</aside>\n")

    def visit_literal_block(self, node: Element):
        """Begin ``literal_block`` .

        Override base method, and open simply ``pre`` and ``code`` tags.
        """
        lang = node["language"]
        # add section id as data-id if it is exists
        if "data-id" in node:
            self.body.append(f'<pre data-id="{node["data-id"]}">')
        elif isinstance(node.parent, section) and len(node.parent["ids"]):
            self.body.append(f'<pre data-id="{node.parent["ids"][0]}">')
        else:
            self.body.append("<pre>")
        self.body.append(f'<code data-trim data-noescape class="{lang}"')
        # use the emphasize-lines directive to create line for line animations
        if "data-line-numbers" in node:
            self.body.append(f' data-line-numbers="{node["data-line-numbers"]}"')
        else:
            # show line numbers
            if node["linenos"]:
                self.body.append(" data-line-numbers")
        if "data-ln-start-from" in node:
            self.body.append(f' data-ln-start-from="{node["data-ln-start-from"]}"')
            if "data-line-numbers" not in node:
                self.body.append(" data-line-numbers")
        self.body.append(">")

    def depart_literal_block(self, node: literal_block):
        """End ``literal_block``.

        Override base method, and close begun tags.
        """
        self.body.append("</code></pre>\n")


def not_write(self, node):
    """visit/depart function for declare "no write"."""
    pass


def visit_revealjs_break(self, node: revealjs_break):
    """Close current section."""
    self.body.append("</section>\n")


def depart_revealjs_break(self, node: revealjs_break):
    """Open as next section.

    If node does not have attribute 'notitle',
    render title from current original section.
    """
    attrs = node.attributes_str()
    self.body.append(f"<section {attrs}>\n")
    if "notitle" not in node.attributes:
        idx = node.parent.first_child_matching_class(title)
        if idx is None:
            return
        # NOTE: It has possibility to work side effect because re-walk for used nodes.
        node.parent.children[idx].walkabout(self)
        self.body.append("\n")



"""Theme handle module."""

from pathlib import Path

DEFAULT_THEME = "revealjs-basic"


def get_theme_path(name: str = DEFAULT_THEME) -> Path:
    """Return directory real path of itself."""
    here = Path(__file__).parent
    return here / name



"""Deprecated notes."""

import sys
import textwrap

from packaging import version
from sphinx.application import Sphinx
from sphinx.config import Config
from sphinx.util import logging

logger = logging.getLogger(__name__)


def _get_python_version() -> version.Version:
    """Retrieve version object of Python."""
    text = f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"
    return version.parse(text)


def _get_sphinx_version() -> version.Version:
    """Retrieve version object of Sphinx."""
    from sphinx import __version__

    return version.parse(__version__)


def _to_single_message(msg: str) -> str:
    """Re-format from triple-quoted string to single-line string."""
    return textwrap.dedent(msg).strip().replace("\n", " ")


def handle(app: Sphinx, config: Config):
    """Monitor dependencies and log deprecated messages."""
    return
    # NOTE: Keep work to warn in next steps.
    python_version = _get_python_version()
    sphinx_version = _get_sphinx_version()
    # For python supporting
    if python_version < version.parse("3.9"):
        msg = f"""
            2024-05-06 - All Python <3.8 is already dropped in supporting.
            {__name__} will supports only python 3.8 and newer using `requires-python` metadata with minor version up.
            Please update python to keep using it.
            """  # noqa: E501
        logger.warning(_to_single_message(msg))

    if sphinx_version.major < 7 or (
        sphinx_version.major == 8 and sphinx_version.minor < 3
    ):
        msg = f"""
            2024-05-06 - In near future,
            {__name__} will drop supports for Sphinx of version less than 5 with minor version up.
            Please update dependencies to keep using it.
            """  # noqa: E501
        logger.warning(_to_single_message(msg))



"""Test cases for ``revealjs-break`` directive."""

import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="default")
def test_on_revealjs_builder(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-break.html")
    section_tag = soup.h2.parent.parent
    assert len(section_tag.find_all("section")) == 3
    # title, content1, content2 (splitted)


@pytest.mark.sphinx("revealjs", testroot="default")
def test_top_of_section(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-break-top.html")
    section_tag = soup.h1.parent.parent
    print(section_tag)
    assert len(section_tag.find_all("section")) == 2


@pytest.mark.sphinx("revealjs", testroot="default")
def test_render_all_items_of_head(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-break-decorated.html")
    head = soup.find_all("h3")[-1]
    assert head.text == "Decorated Head"



"""Test cases for sphix_revealjs.ext.footnotes."""

import pytest
from sphinx.testing.util import SphinxTestApp


@pytest.mark.sphinx(
    "revealjs",
    testroot="default",
    confoverrides={
        "extensions": ["sphinx_revealjs", "sphinx_revealjs.ext.footnotes"],
    },
)
def test_skip_included(app: SphinxTestApp, status, warning):  # noqa
    app.build()
    css_path = app.outdir / "_static/sphinx-revealjs/css/footnotes.css"
    assert css_path.exists()


@pytest.mark.sphinx(
    "dirrevealjs",
    testroot="default",
    freshenv=True,
    confoverrides={
        "extensions": ["sphinx_revealjs", "sphinx_revealjs.ext.footnotes"],
    },
)
def test_dirrevealjs(app: SphinxTestApp, status, warning):  # noqa
    app.build()
    css_path = app.outdir / "_static/sphinx-revealjs/css/footnotes.css"
    assert css_path.exists()



"""Configuration is cases for default behavior."""

extensions = [
    "sphinx_revealjs",
]

# To skip toctree
rst_prolog = """
:orphan:
"""

revealjs_html_theme = "revealjs-simple"


def setup(app):
    def _html_page_context(app, pagename, templatename, context, doctree):
        metatags = context.get("metatags", "")
        metatags += "<style></style>"
        context["metatags"] = metatags

    app.connect("html-page-context", _html_page_context)
